@@ -15,7 +15,7 @@ import threading
 
 _backend_loader = PluginLoader("dogpile.cache")
 register_backend = _backend_loader.register
-from . import backends
+from . import backends  # noqa
 
 value_version = 1
 """An integer placed in the :class:`.CachedValue`
@@ -24,6 +24,7 @@ values from a previous, backwards-incompatible version.
 
 """
 
+
 class CacheRegion(object):
     """A front end to a particular cache backend.
 
@@ -156,7 +157,8 @@ class CacheRegion(object):
 
     """
 
-    def __init__(self,
+    def __init__(
+            self,
             name=None,
             function_key_generator=function_key_generator,
             function_multi_key_generator=function_multi_key_generator,
@@ -167,21 +169,20 @@ class CacheRegion(object):
         self.name = name
         self.function_key_generator = function_key_generator
         self.function_multi_key_generator = function_multi_key_generator
-        if key_mangler:
-            self.key_mangler = key_mangler
-        else:
-            self.key_mangler = None
+        self.key_mangler = self._user_defined_key_mangler = key_mangler
         self._hard_invalidated = None
         self._soft_invalidated = None
         self.async_creation_runner = async_creation_runner
 
-    def configure(self, backend,
+    def configure(
+            self, backend,
             expiration_time=None,
             arguments=None,
             _config_argument_dict=None,
             _config_prefix=None,
-            wrap=None
-        ):
+            wrap=None,
+            replace_existing_backend=False,
+    ):
         """Configure a :class:`.CacheRegion`.
 
         The :class:`.CacheRegion` itself
@@ -220,13 +221,21 @@ class CacheRegion(object):
 
             :ref:`changing_backend_behavior`
 
+        :param replace_existing_backend: if True, the existing cache backend
+         will be replaced.  Without this flag, an exception is raised if
+         a backend is already configured.
+
+         .. versionadded:: 0.5.7
+
+
          """
 
-        if "backend" in self.__dict__:
+        if "backend" in self.__dict__ and not replace_existing_backend:
             raise exception.RegionAlreadyConfigured(
-                    "This region is already "
-                    "configured with backend: %s"
-                    % self.backend)
+                "This region is already "
+                "configured with backend: %s.  "
+                "Specify replace_existing_backend=True to replace."
+                % self.backend)
         backend_cls = _backend_loader.load(backend)
         if _config_argument_dict:
             self.backend = backend_cls.from_config_dict(
@@ -239,17 +248,18 @@ class CacheRegion(object):
         if not expiration_time or isinstance(expiration_time, Number):
             self.expiration_time = expiration_time
         elif isinstance(expiration_time, datetime.timedelta):
-            self.expiration_time = int(compat.timedelta_total_seconds(expiration_time))
+            self.expiration_time = int(
+                compat.timedelta_total_seconds(expiration_time))
         else:
             raise exception.ValidationError(
                 'expiration_time is not a number or timedelta.')
 
-        if self.key_mangler is None:
+        if not self._user_defined_key_mangler:
             self.key_mangler = self.backend.key_mangler
 
         self._lock_registry = NameRegistry(self._create_mutex)
 
-        if getattr(wrap,'__iter__', False):
+        if getattr(wrap, '__iter__', False):
             for wrapper in reversed(wrap):
                 self.wrap(wrapper)
 
@@ -270,7 +280,6 @@ class CacheRegion(object):
 
         self.backend = proxy.wrap(self.backend)
 
-
     def _mutex(self, key):
         return self._lock_registry.get(key)
 
@@ -309,12 +318,12 @@ class CacheRegion(object):
         :meth:`.CacheRegion.get_or_create_multi` and
         :meth:`.CacheRegion.get` methods.
 
-        The method
-        supports both "hard" and "soft" invalidation options.  With "hard"
-        invalidation, :meth:`.CacheRegion.get_or_create` will force an immediate
-        regeneration of the value which all getters will wait for.  With
-        "soft" invalidation, subsequent getters will return the "old" value until
-        the new one is available.
+        The method supports both "hard" and "soft" invalidation
+        options.  With "hard" invalidation,
+        :meth:`.CacheRegion.get_or_create` will force an immediate
+        regeneration of the value which all getters will wait for.
+        With "soft" invalidation, subsequent getters will return the
+        "old" value until the new one is available.
 
         Usage of "soft" invalidation requires that the region or the method
         is given a non-None expiration time.
@@ -363,13 +372,14 @@ class CacheRegion(object):
         config_dict = coerce_string_conf(config_dict)
         return self.configure(
             config_dict["%sbackend" % prefix],
-            expiration_time = config_dict.get(
-                                "%sexpiration_time" % prefix, None),
+            expiration_time=config_dict.get(
+                "%sexpiration_time" % prefix, None),
             _config_argument_dict=config_dict,
-            _config_prefix="%sarguments." % prefix
+            _config_prefix="%sarguments." % prefix,
+            wrap=config_dict.get(
+                "%swrap" % prefix, None),
         )
 
-
     @memoized_property
     def backend(self):
         raise exception.RegionNotConfigured(
@@ -443,7 +453,7 @@ class CacheRegion(object):
             key = self.key_mangler(key)
         value = self.backend.get(key)
         value = self._unexpired_value_fn(
-                        expiration_time, ignore_expiration)(value)
+            expiration_time, ignore_expiration)(value)
 
         return value.payload
 
@@ -457,11 +467,12 @@ class CacheRegion(object):
             current_time = time.time()
 
             invalidated = self._hard_invalidated or self._soft_invalidated
+
             def value_fn(value):
                 if value is NO_VALUE:
                     return value
                 elif expiration_time is not None and \
-                      current_time - value.metadata["ct"] > expiration_time:
+                        current_time - value.metadata["ct"] > expiration_time:
                     return NO_VALUE
                 elif invalidated and \
                         value.metadata["ct"] < invalidated:
@@ -512,18 +523,18 @@ class CacheRegion(object):
         backend_values = self.backend.get_multi(keys)
 
         _unexpired_value_fn = self._unexpired_value_fn(
-                            expiration_time, ignore_expiration)
+            expiration_time, ignore_expiration)
         return [
-                    value.payload if value is not NO_VALUE else value
-                    for value in
-                    (
-                        _unexpired_value_fn(value) for value in
-                        backend_values
-                    )
-                ]
+            value.payload if value is not NO_VALUE else value
+            for value in
+            (
+                _unexpired_value_fn(value) for value in
+                backend_values
+            )
+        ]
 
-    def get_or_create(self, key, creator, expiration_time=None,
-                                should_cache_fn=None):
+    def get_or_create(
+            self, key, creator, expiration_time=None, should_cache_fn=None):
         """Return a cached value based on the given key.
 
         If the value does not exist or is considered to be expired
@@ -563,10 +574,10 @@ class CacheRegion(object):
          the expiration time already configured on this :class:`.CacheRegion`
          if not None.   To set no expiration, use the value -1.
 
-        :param should_cache_fn: optional callable function which will receive the
-         value returned by the "creator", and will then return True or False,
-         indicating if the value should actually be cached or not.  If it
-         returns False, the value is still returned, but isn't cached.
+        :param should_cache_fn: optional callable function which will receive
+         the value returned by the "creator", and will then return True or
+         False, indicating if the value should actually be cached or not.  If
+         it returns False, the value is still returned, but isn't cached.
          E.g.::
 
             def dont_cache_none(value):
@@ -587,7 +598,8 @@ class CacheRegion(object):
             :meth:`.CacheRegion.cache_on_arguments` - applies
             :meth:`.get_or_create` to any function using a decorator.
 
-            :meth:`.CacheRegion.get_or_create_multi` - multiple key/value version
+            :meth:`.CacheRegion.get_or_create_multi` - multiple key/value
+             version
 
         """
         orig_key = key
@@ -598,8 +610,9 @@ class CacheRegion(object):
             value = self.backend.get(key)
             if value is NO_VALUE or \
                 value.metadata['v'] != value_version or \
-                    (self._hard_invalidated and
-                    value.metadata["ct"] < self._hard_invalidated):
+                    (
+                        self._hard_invalidated and
+                        value.metadata["ct"] < self._hard_invalidated):
                 raise NeedRegenerationException()
             ct = value.metadata["ct"]
             if self._soft_invalidated:
@@ -623,12 +636,16 @@ class CacheRegion(object):
 
         if expiration_time is None and self._soft_invalidated:
             raise exception.DogpileCacheException(
-                    "Non-None expiration time required "
-                    "for soft invalidation")
+                "Non-None expiration time required "
+                "for soft invalidation")
+
+        if expiration_time == -1:
+            expiration_time = None
 
         if self.async_creation_runner:
             def async_creator(mutex):
-                return self.async_creation_runner(self, orig_key, creator, mutex)
+                return self.async_creation_runner(
+                    self, orig_key, creator, mutex)
         else:
             async_creator = None
 
@@ -640,8 +657,8 @@ class CacheRegion(object):
                 async_creator) as value:
             return value
 
-    def get_or_create_multi(self, keys, creator, expiration_time=None,
-                                should_cache_fn=None):
+    def get_or_create_multi(
+            self, keys, creator, expiration_time=None, should_cache_fn=None):
         """Return a sequence of cached values based on a sequence of keys.
 
         The behavior for generation of values based on keys corresponds
@@ -654,6 +671,13 @@ class CacheRegion(object):
         The method uses the same approach as :meth:`.Region.get_multi`
         and :meth:`.Region.set_multi` to get and set values from the
         backend.
+        
+        If you are using a :class:`.CacheBackend` or :class:`.ProxyBackend` 
+        that modifies values, take note this function invokes 
+        ``.set_multi()`` for newly generated values using the same values it
+        returns to the calling function. A correct implementation of 
+        ``.set_multi()`` will not modify values in-place on the submitted
+        ``mapping`` dict.
 
         :param keys: Sequence of keys to be retrieved.
 
@@ -665,9 +689,9 @@ class CacheRegion(object):
          if not None.   To set no expiration, use the value -1.
 
         :param should_cache_fn: optional callable function which will receive
-         each value returned by the "creator", and will then return True or False,
-         indicating if the value should actually be cached or not.  If it
-         returns False, the value is still returned, but isn't cached.
+         each value returned by the "creator", and will then return True or
+         False, indicating if the value should actually be cached or not.  If
+         it returns False, the value is still returned, but isn't cached.
 
         .. versionadded:: 0.5.0
 
@@ -686,7 +710,7 @@ class CacheRegion(object):
             if value is NO_VALUE or \
                 value.metadata['v'] != value_version or \
                     (self._hard_invalidated and
-                    value.metadata["ct"] < self._hard_invalidated):
+                        value.metadata["ct"] < self._hard_invalidated):
                 # dogpile.core understands a 0 here as
                 # "the value is not available", e.g.
                 # _has_value() will return False.
@@ -710,8 +734,11 @@ class CacheRegion(object):
 
         if expiration_time is None and self._soft_invalidated:
             raise exception.DogpileCacheException(
-                    "Non-None expiration time required "
-                    "for soft invalidation")
+                "Non-None expiration time required "
+                "for soft invalidation")
+
+        if expiration_time == -1:
+            expiration_time = None
 
         mutexes = {}
 
@@ -732,8 +759,8 @@ class CacheRegion(object):
                     gen_value,
                     lambda: get_value(mangled_key),
                     expiration_time,
-                    async_creator=lambda mutex:
-                            async_creator(orig_key, mutex)):
+                    async_creator=lambda mutex: async_creator(orig_key, mutex)
+            ):
                 pass
         try:
             if mutexes:
@@ -762,13 +789,14 @@ class CacheRegion(object):
             for mutex in mutexes.values():
                 mutex.release()
 
-
     def _value(self, value):
         """Return a :class:`.CachedValue` given a value."""
-        return CachedValue(value, {
-                            "ct": time.time(),
-                            "v": value_version
-                        })
+        return CachedValue(
+            value,
+            {
+                "ct": time.time(),
+                "v": value_version
+            })
 
     def set(self, key, value):
         """Place a new value in the cache under the given key."""
@@ -777,7 +805,6 @@ class CacheRegion(object):
             key = self.key_mangler(key)
         self.backend.set(key, self._value(value))
 
-
     def set_multi(self, mapping):
         """Place new values in the cache under the given keys.
 
@@ -788,13 +815,13 @@ class CacheRegion(object):
             return
 
         if self.key_mangler:
-            mapping = dict((self.key_mangler(k), self._value(v))
-                                for k, v in mapping.items())
+            mapping = dict((
+                self.key_mangler(k), self._value(v))
+                for k, v in mapping.items())
         else:
             mapping = dict((k, self._value(v)) for k, v in mapping.items())
         self.backend.set_multi(mapping)
 
-
     def delete(self, key):
         """Remove a value from the cache.
 
@@ -807,7 +834,6 @@ class CacheRegion(object):
 
         self.backend.delete(key)
 
-
     def delete_multi(self, keys):
         """Remove multiple values from the cache.
 
@@ -823,11 +849,12 @@ class CacheRegion(object):
 
         self.backend.delete_multi(keys)
 
-
-    def cache_on_arguments(self, namespace=None,
-                                    expiration_time=None,
-                                    should_cache_fn=None,
-                                    to_str=compat.string_type):
+    def cache_on_arguments(
+            self, namespace=None,
+            expiration_time=None,
+            should_cache_fn=None,
+            to_str=compat.string_type,
+            function_key_generator=None):
         """A function decorator that will cache the return
         value of the function using a key derived from the
         function itself and its arguments.
@@ -866,9 +893,9 @@ class CacheRegion(object):
 
             generate_something.set(3, 5, 6)
 
-        The above example is equivalent to calling ``generate_something(5, 6)``,
-        if the function were to produce the value ``3`` as the value to be
-        cached.
+        The above example is equivalent to calling
+        ``generate_something(5, 6)``, if the function were to produce
+        the value ``3`` as the value to be cached.
 
         .. versionadded:: 0.4.1 Added ``set()`` method to decorated function.
 
@@ -986,6 +1013,12 @@ class CacheRegion(object):
 
          .. versionadded:: 0.5.0
 
+        :param function_key_generator: a function that will produce a
+         "cache key". This function will supersede the one configured on the
+         :class:`.CacheRegion` itself.
+
+         .. versionadded:: 0.5.5
+
         .. seealso::
 
             :meth:`.CacheRegion.cache_multi_on_arguments`
@@ -994,21 +1027,28 @@ class CacheRegion(object):
 
         """
         expiration_time_is_callable = compat.callable(expiration_time)
+
+        if function_key_generator is None:
+            function_key_generator = self.function_key_generator
+
         def decorator(fn):
             if to_str is compat.string_type:
                 # backwards compatible
-                key_generator = self.function_key_generator(namespace, fn)
+                key_generator = function_key_generator(namespace, fn)
             else:
-                key_generator = self.function_key_generator(namespace, fn,
-                                    to_str=to_str)
+                key_generator = function_key_generator(
+                    namespace, fn,
+                    to_str=to_str)
+
             @wraps(fn)
             def decorate(*arg, **kw):
                 key = key_generator(*arg, **kw)
+
                 @wraps(fn)
                 def creator():
                     return fn(*arg, **kw)
                 timeout = expiration_time() if expiration_time_is_callable \
-                            else expiration_time
+                    else expiration_time
                 return self.get_or_create(key, creator, timeout,
                                           should_cache_fn)
 
@@ -1038,9 +1078,11 @@ class CacheRegion(object):
             return decorate
         return decorator
 
-    def cache_multi_on_arguments(self, namespace=None, expiration_time=None,
-                                        should_cache_fn=None,
-                                        asdict=False, to_str=compat.string_type):
+    def cache_multi_on_arguments(
+            self, namespace=None, expiration_time=None,
+            should_cache_fn=None,
+            asdict=False, to_str=compat.string_type,
+            function_multi_key_generator=None):
         """A function decorator that will cache multiple return
         values from the function using a sequence of keys derived from the
         function itself and the arguments passed to it.
@@ -1058,15 +1100,16 @@ class CacheRegion(object):
                 ]
 
         The decorated function can be called normally.  The decorator
-        will produce a list of cache keys using a mechanism similar to that
-        of :meth:`.CacheRegion.cache_on_arguments`, combining the name
-        of the function with the optional namespace and with the string form
-        of each key.  It will then consult the cache using the same mechanism as that
-        of :meth:`.CacheRegion.get_multi` to retrieve all current values;
-        the originally passed keys corresponding to those values which
-        aren't generated or need regeneration will
-        be assembled into a new argument list, and the decorated function
-        is then called with that subset of arguments.
+        will produce a list of cache keys using a mechanism similar to
+        that of :meth:`.CacheRegion.cache_on_arguments`, combining the
+        name of the function with the optional namespace and with the
+        string form of each key.  It will then consult the cache using
+        the same mechanism as that of :meth:`.CacheRegion.get_multi`
+        to retrieve all current values; the originally passed keys
+        corresponding to those values which aren't generated or need
+        regeneration will be assembled into a new argument list, and
+        the decorated function is then called with that subset of
+        arguments.
 
         The returned result is a list::
 
@@ -1077,10 +1120,10 @@ class CacheRegion(object):
         cache and conditionally call the function.  See that
         method for additional behavioral details.
 
-        Unlike the :meth:`.CacheRegion.cache_on_arguments`
-        method, :meth:`.CacheRegion.cache_multi_on_arguments` works only
-        with a single function signature, one which takes a simple list of keys as
-        arguments.
+        Unlike the :meth:`.CacheRegion.cache_on_arguments` method,
+        :meth:`.CacheRegion.cache_multi_on_arguments` works only with
+        a single function signature, one which takes a simple list of
+        keys as arguments.
 
         Like :meth:`.CacheRegion.cache_on_arguments`, the decorated function
         is also provided with a ``set()`` method, which here accepts a
@@ -1119,9 +1162,10 @@ class CacheRegion(object):
          expiration time.  May be passed as an integer or a
          callable.
 
-        :param should_cache_fn: passed to :meth:`.CacheRegion.get_or_create_multi`.
-         This function is given a value as returned by the creator, and only
-         if it returns True will that value be placed in the cache.
+        :param should_cache_fn: passed to
+         :meth:`.CacheRegion.get_or_create_multi`. This function is given a
+         value as returned by the creator, and only if it returns True will
+         that value be placed in the cache.
 
         :param asdict: if ``True``, the decorated function should return
          its result as a dictionary of keys->values, and the final result
@@ -1142,6 +1186,12 @@ class CacheRegion(object):
 
         .. versionadded:: 0.5.0
 
+        :param function_multi_key_generator: a function that will produce a
+         list of keys. This function will supersede the one configured on the
+         :class:`.CacheRegion` itself.
+
+         .. versionadded:: 0.5.5
+
         .. seealso::
 
             :meth:`.CacheRegion.cache_on_arguments`
@@ -1150,25 +1200,34 @@ class CacheRegion(object):
 
         """
         expiration_time_is_callable = compat.callable(expiration_time)
+
+        if function_multi_key_generator is None:
+            function_multi_key_generator = self.function_multi_key_generator
+
         def decorator(fn):
-            key_generator = self.function_multi_key_generator(namespace, fn,
-                                            to_str=to_str)
+            key_generator = function_multi_key_generator(
+                namespace, fn,
+                to_str=to_str)
+
             @wraps(fn)
             def decorate(*arg, **kw):
                 cache_keys = arg
                 keys = key_generator(*arg, **kw)
                 key_lookup = dict(zip(keys, cache_keys))
+
                 @wraps(fn)
                 def creator(*keys_to_create):
                     return fn(*[key_lookup[k] for k in keys_to_create])
 
                 timeout = expiration_time() if expiration_time_is_callable \
-                            else expiration_time
+                    else expiration_time
 
                 if asdict:
                     def dict_create(*keys):
                         d_values = creator(*keys)
-                        return [d_values.get(key_lookup[k], NO_VALUE) for k in keys]
+                        return [
+                            d_values.get(key_lookup[k], NO_VALUE)
+                            for k in keys]
 
                     def wrap_cache_fn(value):
                         if value is NO_VALUE:
@@ -1178,13 +1237,15 @@ class CacheRegion(object):
                         else:
                             return should_cache_fn(value)
 
-                    result = self.get_or_create_multi(keys, dict_create, timeout,
-                                         wrap_cache_fn)
-                    result = dict((k, v) for k, v in zip(cache_keys, result)
-                                        if v is not NO_VALUE)
+                    result = self.get_or_create_multi(
+                        keys, dict_create, timeout, wrap_cache_fn)
+                    result = dict(
+                        (k, v) for k, v in zip(cache_keys, result)
+                        if v is not NO_VALUE)
                 else:
-                    result = self.get_or_create_multi(keys, creator, timeout,
-                                          should_cache_fn)
+                    result = self.get_or_create_multi(
+                        keys, creator, timeout,
+                        should_cache_fn)
 
                 return result
 
@@ -1196,10 +1257,10 @@ class CacheRegion(object):
                 keys = list(mapping)
                 gen_keys = key_generator(*keys)
                 self.set_multi(dict(
-                        (gen_key, mapping[key])
-                        for gen_key, key
-                        in zip(gen_keys, keys))
-                    )
+                    (gen_key, mapping[key])
+                    for gen_key, key
+                    in zip(gen_keys, keys))
+                )
 
             def get(*arg):
                 keys = key_generator(*arg)
@@ -1210,13 +1271,13 @@ class CacheRegion(object):
                 values = fn(*arg)
                 if asdict:
                     self.set_multi(
-                                dict(zip(keys, [values[a] for a in arg]))
-                            )
+                        dict(zip(keys, [values[a] for a in arg]))
+                    )
                     return values
                 else:
                     self.set_multi(
-                                dict(zip(keys, values))
-                            )
+                        dict(zip(keys, values))
+                    )
                     return values
 
             decorate.set = set_
@@ -1228,7 +1289,6 @@ class CacheRegion(object):
         return decorator
 
 
-
 def make_region(*arg, **kw):
     """Instantiate a new :class:`.CacheRegion`.
 
