@@ -1,5 +1,5 @@
 import re
-from nose import SkipTest
+import pytest
 from functools import wraps
 from dogpile.cache import compat
 import time
@@ -9,14 +9,17 @@ def eq_(a, b, msg=None):
     """Assert a == b, with repr messaging on failure."""
     assert a == b, msg or "%r != %r" % (a, b)
 
+
 def is_(a, b, msg=None):
     """Assert a is b, with repr messaging on failure."""
     assert a is b, msg or "%r is not %r" % (a, b)
 
+
 def ne_(a, b, msg=None):
     """Assert a != b, with repr messaging on failure."""
     assert a != b, msg or "%r == %r" % (a, b)
 
+
 def assert_raises_message(except_cls, msg, callable_, *args, **kwargs):
     try:
         callable_(*args, **kwargs)
@@ -24,7 +27,8 @@ def assert_raises_message(except_cls, msg, callable_, *args, **kwargs):
     except except_cls as e:
         assert re.search(msg, str(e)), "%r !~ %s" % (msg, e)
 
-from dogpile.cache.compat import configparser, io
+from dogpile.cache.compat import configparser, io  # noqa
+
 
 def winsleep():
     # sleep a for an amount of time
@@ -33,10 +37,11 @@ def winsleep():
     if compat.win32:
         time.sleep(.001)
 
+
 def requires_py3k(fn):
     @wraps(fn)
     def wrap(*arg, **kw):
         if compat.py2k:
-            raise SkipTest("Python 3 required")
+            pytest.skip("Python 3 required")
         return fn(*arg, **kw)
-    return wrap
\ No newline at end of file
+    return wrap
