@@ -1,13 +1,16 @@
 from ._fixtures import _GenericBackendTest, _GenericMutexTest
-from . import eq_, winsleep
+from . import eq_
 from unittest import TestCase
 from threading import Thread
 import time
-from nose import SkipTest
+import pytest
 from dogpile.cache import compat
 
+LOCK_TIMEOUT = 1
+
 
 class _TestMemcachedConn(object):
+
     @classmethod
     def _check_backend_available(cls, backend):
         try:
@@ -15,10 +18,11 @@ class _TestMemcachedConn(object):
             client.set("x", "y")
             assert client.get("x") == "y"
         except:
-            raise SkipTest(
+            pytest.skip(
                 "memcached is not running or "
                 "otherwise not functioning correctly")
 
+
 class _NonDistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
     region_args = {
         "key_mangler": lambda x: x.replace(" ", "_")
@@ -29,6 +33,20 @@ class _NonDistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
         }
     }
 
+
+class _DistributedMemcachedWithTimeoutTest(_TestMemcachedConn, _GenericBackendTest):
+    region_args = {
+        "key_mangler": lambda x: x.replace(" ", "_")
+    }
+    config_args = {
+        "arguments": {
+            "url": "127.0.0.1:11211",
+            "distributed_lock": True,
+            "lock_timeout": LOCK_TIMEOUT,
+        }
+    }
+
+
 class _DistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
     region_args = {
         "key_mangler": lambda x: x.replace(" ", "_")
@@ -36,10 +54,11 @@ class _DistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
     config_args = {
         "arguments": {
             "url": "127.0.0.1:11211",
-            "distributed_lock": True
+            "distributed_lock": True,
         }
     }
 
+
 class _DistributedMemcachedMutexTest(_TestMemcachedConn, _GenericMutexTest):
     config_args = {
         "arguments": {
@@ -48,30 +67,57 @@ class _DistributedMemcachedMutexTest(_TestMemcachedConn, _GenericMutexTest):
         }
     }
 
+
+class _DistributedMemcachedMutexWithTimeoutTest(_TestMemcachedConn, _GenericMutexTest):
+    config_args = {
+        "arguments": {
+            "url": "127.0.0.1:11211",
+            "distributed_lock": True,
+            "lock_timeout": LOCK_TIMEOUT,
+        }
+    }
+
+
 class PylibmcTest(_NonDistributedMemcachedTest):
     backend = "dogpile.cache.pylibmc"
 
+
 class PylibmcDistributedTest(_DistributedMemcachedTest):
     backend = "dogpile.cache.pylibmc"
 
+
 class PylibmcDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.pylibmc"
 
+
 class BMemcachedTest(_NonDistributedMemcachedTest):
     backend = "dogpile.cache.bmemcached"
 
+
+class BMemcachedDistributedWithTimeoutTest(_DistributedMemcachedWithTimeoutTest):
+    backend = "dogpile.cache.bmemcached"
+
+
 class BMemcachedDistributedTest(_DistributedMemcachedTest):
     backend = "dogpile.cache.bmemcached"
 
+
 class BMemcachedDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.bmemcached"
 
+
+class BMemcachedDistributedMutexWithTimeoutTest(_DistributedMemcachedMutexWithTimeoutTest):
+    backend = "dogpile.cache.bmemcached"
+
+
 class MemcachedTest(_NonDistributedMemcachedTest):
     backend = "dogpile.cache.memcached"
 
+
 class MemcachedDistributedTest(_DistributedMemcachedTest):
     backend = "dogpile.cache.memcached"
 
+
 class MemcachedDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.memcached"
 
@@ -80,29 +126,36 @@ from dogpile.cache.backends.memcached import GenericMemcachedBackend
 from dogpile.cache.backends.memcached import PylibmcBackend
 from dogpile.cache.backends.memcached import MemcachedBackend
 
+
 class MockGenericMemcachedBackend(GenericMemcachedBackend):
+
     def _imports(self):
         pass
 
     def _create_client(self):
         return MockClient(self.url)
 
+
 class MockMemcacheBackend(MemcachedBackend):
+
     def _imports(self):
         pass
 
     def _create_client(self):
         return MockClient(self.url)
 
+
 class MockPylibmcBackend(PylibmcBackend):
+
     def _imports(self):
         pass
 
     def _create_client(self):
         return MockClient(self.url,
-                        binary=self.binary,
-                        behaviors=self.behaviors
-                    )
+                          binary=self.binary,
+                          behaviors=self.behaviors
+                          )
+
 
 class MockClient(object):
     number_of_clients = 0
@@ -116,17 +169,22 @@ class MockClient(object):
 
     def get(self, key):
         return self._cache.get(key)
+
     def set(self, key, value, **kw):
         self.canary.append(kw)
         self._cache[key] = value
+
     def delete(self, key):
         self._cache.pop(key, None)
+
     def __del__(self):
         MockClient.number_of_clients -= 1
 
+
 class PylibmcArgsTest(TestCase):
+
     def test_binary_flag(self):
-        backend = MockPylibmcBackend(arguments={'url': 'foo','binary': True})
+        backend = MockPylibmcBackend(arguments={'url': 'foo', 'binary': True})
         eq_(backend._create_client().kw["binary"], True)
 
     def test_url_list(self):
@@ -139,18 +197,18 @@ class PylibmcArgsTest(TestCase):
 
     def test_behaviors(self):
         backend = MockPylibmcBackend(arguments={'url': "foo",
-                                    "behaviors": {"q": "p"}})
+                                                "behaviors": {"q": "p"}})
         eq_(backend._create_client().kw["behaviors"], {"q": "p"})
 
     def test_set_time(self):
         backend = MockPylibmcBackend(arguments={'url': "foo",
-                                "memcached_expire_time": 20})
+                                                "memcached_expire_time": 20})
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"time": 20}])
 
     def test_set_min_compress_len(self):
         backend = MockPylibmcBackend(arguments={'url': "foo",
-                                "min_compress_len": 20})
+                                                "min_compress_len": 20})
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"min_compress_len": 20}])
 
@@ -159,21 +217,24 @@ class PylibmcArgsTest(TestCase):
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{}])
 
+
 class MemcachedArgstest(TestCase):
+
     def test_set_time(self):
         backend = MockMemcacheBackend(arguments={'url': "foo",
-                                "memcached_expire_time": 20})
+                                                 "memcached_expire_time": 20})
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"time": 20}])
 
     def test_set_min_compress_len(self):
         backend = MockMemcacheBackend(arguments={'url': "foo",
-                                "min_compress_len": 20})
+                                                 "min_compress_len": 20})
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"min_compress_len": 20}])
 
 
 class LocalThreadTest(TestCase):
+
     def setUp(self):
         import gc
         gc.collect()
@@ -208,5 +269,3 @@ class LocalThreadTest(TestCase):
             eq_(MockClient.number_of_clients, 0)
         else:
             eq_(MockClient.number_of_clients, 1)
-
-
